import json

class AppNotDefined(Exception):
    """All routes will fail if app is not defined"""
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message

class HttpMethodConflict(Exception):
    """Defined HTTP methods with defined function once"""
    def __init__(self, message):
        self.message = message

    def __str__(self):
        return self.message

class Froi:
    """
    Template for all routes to be created for APIs.

    A Flask wrapper that installs routes as defined from a class.
    It will accept :app: as paremeter that will be the server's
    context and will be used to set route.
    """

    def __init__(self, app, component_name, prefix=''):
        self.component_name = component_name
        self.prefix = prefix
        if app.add_url_rule is None:
            raise AppNotDefined('Sent `app` is not valid')

        self.app = app
        self.method = None

    def route(self, url='', **kwargs):
        """Wrap server method's route"""
        add = self.app.add_url_rule
        add('{}{}'.format(self.prefix, url),
            methods=self.method,
            **kwargs)

        # handle forward slash
        if self.prefix is not '' or url is not '':
            add('{}{}/'.format(self.prefix, url),
                methods=self.method,
                **kwargs)

        self.method = None

    def _master_fxn(self, **kwargs):
        from flask import request
        return json.dumps({
            'message': '{} on {}'.format(request.method, self.component_name)
        })

    def install(self):
        """Attach routes to defined app"""
        self.all().route(view_func=self._master_fxn)

    def _check_methods(self, method):
        if self.method is not None:
            self.method += [method]
            return

        self.method = [method]

    def all(self):
        self.get().post().put().delete()
        return self

    def get(self):
        self._check_methods('GET')
        return self

    def post(self):
        self._check_methods('POST')
        return self

    def put(self):
        self._check_methods('PUT')
        return self

    def delete(self):
        self._check_methods('DELETE')
        return self



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['froi']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'froi',
    'version': '0.1.2',
    'description': 'A Flask wrapper to easily handle routes defined as Python native objects',
    'long_description': "Froi (Flask Router Object Interface)\n====================================\n\nA wrapper for Flask's native routing as a form of template.\nRoutes will be defined as object to easily define domains.\n\n.. code-block:: python\n\n    # Inside some route object\n    from froi import Froi\n    class SomeRoute(Froi):\n        def __init__(self, app):\n            super().__init__(app, 'SomeRoute', '/some_route')\n\n        def install(self):\n            # define a get route on base\n            self.get().route(view_func=sample_fxn_1)\n\n            # define a post\n            self.post().route('/do_something', view_func=sample_fxn_2)\n\n            # define a put\n            self.put().route('/edit_something', view_func=sample_fxn_3)\n\n            # define a delete\n            self.delete().route('/delete_something', view_func=sample_fxn_4)\n\n    # Inside your server handler\n    from flask import Flask\n    from some_route import SomeRoute\n    app = Flask(config.APP_NAME)\n    SomeRoute(app).install()\n    app.run()\n\nIf you want a RESTful pattern to handle the routes, you can omit defining the _install_ function.\n\n.. code-block:: python\n\n    from froi import Froi\n    class SomeRoute(Froi):\n        def __init__(self, app):\n            super().__init__(app, 'SomeRoute', '/some_route')\n\nThis will automatically create a `GET`, `POST`, `PUT`, and `DELETE` endpoints.\n\nThis feature though is a work in progress and may not work as expected.\n",
    'author': 'Almer Mendoza',
    'author_email': 'atmalmer23@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
